# Prompt the user to enter the type of drive they are planning to use this on
$driveType = Read-Host "Please enter the type of drive you are planning to use this on (SSD or HDD)"

# Validate the drive type input
if ($driveType -eq "SSD" -or $driveType -eq "HDD") {
    # Proceed with the script
} else {
    # Display an error message and exit the script
    Write-Error "Invalid drive type. Please enter either SSD or HDD."
    Exit
}

# If the drive type is SSD, inform the user of the pros of compression
if ($driveType -eq "SSD") {
    Write-Host "Compressing files and folders on a solid state drive (SSD) can have some benefits, such as:"
    Write-Host "- You can save some disk space by compressing files and folders that are not already compressed or optimized. This can help you store more data on your SSD or free up some space for other purposes."
    Write-Host "- You can improve the performance of your SSD by reducing the amount of data that needs to be read and written. This can lower the wear and tear of your SSD and extend its lifespan. It can also reduce the power consumption and heat generation of your SSD, which can benefit your battery life and system stability."
    Write-Host "- You can enhance the security of your data by making it harder for unauthorized users or programs to access or modify your files. Compression can also help you recover some data in case of corruption or deletion, as it may preserve some parts of the original file."
    Write-Host "However, there are also some cons of using this script on an SSD, such as:"
    Write-Host "- You may experience some CPU overhead when compressing and decompressing files, which can affect your system performance and responsiveness. This may not be noticeable on modern CPUs, but it can still have some impact on your multitasking and gaming experience."
    Write-Host "- You may encounter some compatibility issues with some applications or games that do not support compressed files or folders. This can cause errors, crashes, or performance degradation. You may need to decompress some files or folders before using them with certain programs."
    Write-Host "- You may lose some data quality or functionality when compressing some types of files, such as audio, video, or document files. Compression can reduce the resolution, bitrate, or features of these files, which can affect their appearance, sound, or behavior."
    Write-Host "Therefore, you should weigh the pros and cons of using this script on an SSD before deciding to do so. You should also backup your important data before running the script, in case something goes wrong. You can read more about how compression works on SSDs in [this article](^3^) or this question."
}

# If the drive type is HDD, inform the user of the benefits of compression
if ($driveType -eq "HDD") {
    Write-Host "Compressing files and folders on a hard disk drive (HDD) can improve performance by reducing disk space usage, increasing data transfer speed, and reducing seek time. However, compression also increases CPU usage and may affect some applications or games. You can read more about the pros and cons of compression on HDDs in [this article](^2^)."
}

# Prompt the user to enter a drive letter for compression
$drive = Read-Host "Please enter a drive letter for compression (e.g. C, D, E)"

# Validate the drive letter input
if ($drive -match "^[A-Z]$") {
    # Append a colon to the drive letter
    $drive = $drive + ":"
} else {
    # Display an error message and exit the script
    Write-Error "Invalid drive letter. Please enter a single uppercase letter from A to Z."
    Exit
}

# Define the file types to exclude from compression
$exclude = @("*.zip", "*.mp4", "*.mp3", "*.flv", "*.jpg", "*.png", "*.gif", "*.pdf", "*.rar", "*.7z", "*.tar", "*.gz", "*.wav", "*.aac", "*.ogg", "*.mkv", "*.avi", "*.docx", "*.xlsx", "*.pptx", "*.ttf", "*.otf", "*.woff")

# Get all the files and folders on the drive
$items = Get-ChildItem -Path $drive -Recurse

# Loop through each item and check if it is compressible
foreach ($item in $items) {
    # Skip the item if it is a directory or a symbolic link
    if ($item.PSIsContainer -or $item.LinkType) {
        continue
    }

    # Skip the item if it matches any of the excluded file types
    $skip = $false
    foreach ($ext in $exclude) {
        if ($item.Name -like $ext) {
            $skip = $true
            break
        }
    }
    if ($skip) {
        continue
    }

    # Compress the item using compact.exe with the /C, /I, /EXE:lzx and /EXCLUDE:exclude.txt options
    compact /C /I /EXE:lzx /EXCLUDE:exclude.txt $item.FullName
}

# Prompt the user if they wish to undo compression in case of compatibility reasons
$undo = Read-Host "Do you wish to undo compression in case of compatibility reasons? (Y/N)"

# Validate the undo input
if ($undo -eq "Y" -or $undo -eq "N") {
    # Proceed with the script
} else {
    # Display an error message and exit the script
    Write-Error "Invalid input. Please enter either Y or N."
    Exit
}

# If the user wishes to undo compression, run the command to uncompress the data
if ($undo -eq "Y") {
    Write-Host "Uncompressing the data. This may take some time."
    compact.exe /CompactOS:never
    Write-Host "The data has been uncompressed. You may need to restart your system for the changes to take effect."
}

# If the user does not wish to undo compression, display a message and exit the script
if ($undo -eq "N") {
    Write-Host "The data has been compressed. You may need to restart your system for the changes to take effect."
}
